@@ -27,9 +27,9 @@ type Context struct {
 
 	Globals Symtab
 
-	//  UnaryFn maps the names of unary functions (ops) to their implemenations.
+	//  UnaryFn maps the names of unary functions (ops) to their implementations.
 	UnaryFn map[string]*Function
-	//  BinaryFn maps the names of binary functions (ops) to their implemenations.
+	//  BinaryFn maps the names of binary functions (ops) to their implementations.
 	BinaryFn map[string]*Function
 	// Defs is a list of defined ops, in time order.  It is used when saving the
 	// Context to a file.
@@ -74,14 +74,26 @@ func (c *Context) Local(i int) value.Value {
 }
 
 // AssignLocal assigns the local variable with the given index the value.
-func (c *Context) AssignLocal(i int, value value.Value) {
-	c.stack[len(c.stack)-i] = value
+func (c *Context) AssignLocal(i int, val value.Value) {
+	switch v := val.(type) {
+	case value.Vector:
+		val = v.Copy()
+	case *value.Matrix:
+		val = v.Copy()
+	}
+	c.stack[len(c.stack)-i] = val
 }
 
 // Assign assigns the global variable the value. The variable must
 // be defined either in the current function or globally.
 // Inside a function, new variables become locals.
 func (c *Context) AssignGlobal(name string, val value.Value) {
+	switch v := val.(type) {
+	case value.Vector:
+		val = v.Copy()
+	case *value.Matrix:
+		val = v.Copy()
+	}
 	c.Globals[name] = val
 }
 
@@ -122,6 +134,15 @@ func (c *Context) EvalUnary(op string, right value.Value) value.Value {
 			return value.Reduce(c, op[:len(op)-1], right)
 		case '\\':
 			return value.Scan(c, op[:len(op)-1], right)
+		case '%':
+			if len(op) > 2 {
+				switch op[len(op)-2] {
+				case '/':
+					return value.ReduceFirst(c, op[:len(op)-2], right)
+				case '\\':
+					return value.ScanFirst(c, op[:len(op)-2], right)
+				}
+			}
 		}
 	}
 	fn := c.Unary(op)
@@ -152,6 +173,14 @@ func (c *Context) UserDefined(op string, isBinary bool) bool {
 
 // EvalBinary evaluates a binary operator, including products.
 func (c *Context) EvalBinary(left value.Value, op string, right value.Value) value.Value {
+	// Special handling for the equal and non-equal operators, which must avoid
+	// type conversions involving Char.
+	if op == "==" || op == "!=" {
+		v, ok := value.EvalCharEqual(left, op == "==", right)
+		if ok {
+			return v
+		}
+	}
 	if strings.Contains(op, ".") {
 		return value.Product(c, left, op, right)
 	}
@@ -204,6 +233,27 @@ func (c *Context) Define(fn *Function) {
 	c.Defs = append(c.Defs, OpDef{fn.Name, fn.IsBinary})
 }
 
+func (c *Context) Undefine(name string, binary bool) {
+	// Is it already defined?
+	for i, def := range c.Defs {
+		if def.Name == name && def.IsBinary == binary {
+			// Yes. Drop it.
+			c.Defs = append(c.Defs[:i], c.Defs[i+1:]...)
+			if binary {
+				delete(c.BinaryFn, name)
+			} else {
+				delete(c.UnaryFn, name)
+			}
+			return
+		}
+	}
+	ary := "unary"
+	if binary {
+		ary = "binary"
+	}
+	value.Errorf("no definition for %s %s", ary, name)
+}
+
 // noVar guarantees that there is no global variable with that name,
 // preventing an op from being defined with the same name as a variable,
 // which could cause problems. A variable with value zero is considered to
