@@ -28,9 +28,9 @@ type Token struct {
 type Type int
 
 const (
-	EOF   Type = iota
-	Error      // error occurred; value is text of error
-	Newline
+	EOF     Type = iota
+	Error        // error occurred; value is text of error
+	Newline      // includes spaces and comments leading up to newline.
 	// Interesting things
 	Assign     // '='
 	Char       // printable ASCII character; grab bag for comma etc.
@@ -40,6 +40,7 @@ const (
 	Number     // simple number
 	Operator   // known operator
 	Op         // "op", operator definition keyword
+	OpDelete   // "opdelete", operator undefinition keyword
 	Rational   // rational number like 2/3
 	Complex    // complex number like 3j2
 	RightBrack // ']'
@@ -142,9 +143,11 @@ func (l *Scanner) peek() rune {
 // peek2 returns the next two runes ahead, but does not consume anything.
 func (l *Scanner) peek2() (rune, rune) {
 	pos := l.pos
+	lastWidth := l.lastWidth
 	r1 := l.next()
 	r2 := l.next()
 	l.pos = pos
+	l.lastWidth = lastWidth
 	return r1, r2
 }
 
@@ -156,7 +159,7 @@ func (l *Scanner) backup() {
 	if l.pos == l.start {
 		l.errorf("internal error: backup at start of input")
 	}
-	if l.pos > l.start { // TODO can't happen?
+	if l.pos > l.start {
 		l.pos -= l.lastWidth
 	}
 }
@@ -239,7 +242,6 @@ func lexComment(l *Scanner) stateFn {
 	}
 	if len(l.input) > 0 {
 		l.pos = len(l.input)
-		l.start = l.pos - 1
 		// Emitting newline also advances l.line.
 		return l.emit(Newline)
 	}
@@ -338,6 +340,8 @@ func lexIdentifier(l *Scanner) stateFn {
 	switch {
 	case word == "op":
 		return l.emit(Op)
+	case word == "opdelete":
+		return l.emit(OpDelete)
 	case word == "o" && l.peek() == '.':
 		return lexOperator
 	case l.defined(word):
@@ -357,12 +361,12 @@ func lexOperator(l *Scanner) stateFn {
 	word := l.input[l.start:l.pos]
 	if word == "o" || value.BinaryOps[word] != nil || l.context.UserDefined(word, true) {
 		switch l.peek() {
-		case '/':
-			// Reduction.
-			l.next()
-		case '\\':
-			// Scan.
+		case '/', '\\':
+			// Reduction or scan.
 			l.next()
+			if l.peek() == '%' {
+				l.next()
+			}
 		case '.':
 			// Inner or outer product?
 			l.next()               // Accept the '.'.
@@ -442,6 +446,9 @@ func acceptNumber(l *Scanner, realPart bool) (bool, stateFn) {
 		// Might be a scan or reduction.
 		if r == '/' || r == '\\' {
 			l.next()
+			if l.peek() == '%' {
+				l.next()
+			}
 			return false, l.emit(Operator)
 		}
 		if r != '.' && !l.isNumeral(r) {
@@ -638,7 +645,7 @@ func (l *Scanner) isNumeral(r rune) bool {
 }
 
 // isAllDigits reports whether s consists of digits in the specified base,
-// includig possibly one 'j'.
+// including possibly one 'j'.
 func isAllDigits(s string, base int) bool {
 	top := 'a' + rune(base-10) - 1
 	TOP := 'A' + rune(base-10) - 1
@@ -666,8 +673,12 @@ func isAllDigits(s string, base int) bool {
 // if it is a two-character operator.
 func (l *Scanner) isOperator(r rune) bool {
 	switch r {
-	case '?', '+', '-', '/', '%', '&', '|', '^', ',':
+	case '?', '+', '-', '/', '&', '|', '^':
 		// No follow-on possible.
+	case ',':
+		if l.peek() == '%' {
+			l.next()
+		}
 	case '!':
 		if l.peek() == '=' {
 			l.next()
@@ -683,8 +694,7 @@ func (l *Scanner) isOperator(r rune) bool {
 			l.next()
 		}
 	case '*':
-		switch l.peek() {
-		case '*':
+		if l.peek() == '*' {
 			l.next()
 		}
 	case '=':
