@@ -122,6 +122,17 @@ func (f BigFloat) Sprint(conf *config.Config) string {
 	return f.Float.Text(verb, prec)
 }
 
+// inverse returns 1/f
+func (f BigFloat) inverse() Value {
+	if f.Sign() == 0 {
+		Errorf("inverse of zero")
+	}
+	one := floatOne.Copy(floatOne)
+	return BigFloat{
+		Float: one.Quo(one, f.Float),
+	}.shrink()
+}
+
 func (f BigFloat) ProgString() string {
 	// There is no such thing as a float literal in program listings.
 	panic("float.ProgString - cannot happen")
@@ -131,6 +142,12 @@ func (f BigFloat) Eval(Context) Value {
 	return f
 }
 
+func (f BigFloat) Copy() Value {
+	var x big.Float
+	x.Set(f.Float)
+	return BigFloat{&x}
+}
+
 func (f BigFloat) Inner() Value {
 	return f
 }
@@ -140,9 +157,9 @@ func (f BigFloat) toType(op string, conf *config.Config, which valueType) Value
 	case bigFloatType:
 		return f
 	case complexType:
-		return newComplex(f, Int(0))
+		return NewComplex(f, zero)
 	case vectorType:
-		return NewVector([]Value{f})
+		return oneElemVector(f)
 	case matrixType:
 		return NewMatrix([]int{1}, []Value{f})
 	}
