@@ -130,10 +130,26 @@ func eExponent(x *big.Int) int {
 	return e
 }
 
+// inverse returns 1/i
+func (i BigInt) inverse() Value {
+	if i.Sign() == 0 {
+		Errorf("inverse of zero")
+	}
+	return BigRat{
+		Rat: big.NewRat(0, 1).SetFrac(bigIntOne.Int, i.Int),
+	}.shrink()
+}
+
 func (i BigInt) Eval(Context) Value {
 	return i
 }
 
+func (i BigInt) Copy() Value {
+	var x big.Int
+	x.Set(i.Int)
+	return BigInt{&x}
+}
+
 func (i BigInt) Inner() Value {
 	return i
 }
@@ -149,9 +165,9 @@ func (i BigInt) toType(op string, conf *config.Config, which valueType) Value {
 		f := new(big.Float).SetPrec(conf.FloatPrec()).SetInt(i.Int)
 		return BigFloat{f}
 	case complexType:
-		return newComplex(i, Int(0))
+		return NewComplex(i, zero)
 	case vectorType:
-		return NewVector([]Value{i})
+		return oneElemVector(i)
 	case matrixType:
 		return NewMatrix([]int{1}, []Value{i})
 	}
@@ -192,6 +208,6 @@ func (i BigInt) BitLen() int64 {
 func mustFit(conf *config.Config, n int64) {
 	max := conf.MaxBits()
 	if max != 0 && n > int64(max) {
-		Errorf("result too large (%d bits)", n)
+		Errorf("result too large (%d bits, max %d)", n, conf.MaxBits())
 	}
 }
