@@ -163,10 +163,26 @@ func ratScale(x *big.Rat, exp int) {
 	}
 }
 
+// inverse returns 1/r
+func (r BigRat) inverse() Value {
+	if r.Sign() == 0 {
+		Errorf("inverse of zero")
+	}
+	return BigRat{
+		Rat: big.NewRat(0, 1).SetFrac(r.Denom(), r.Num()),
+	}.shrink()
+}
+
 func (r BigRat) Eval(Context) Value {
 	return r
 }
 
+func (r BigRat) Copy() Value {
+	var x big.Rat
+	x.Set(r.Rat)
+	return BigRat{&x}
+}
+
 func (r BigRat) Inner() Value {
 	return r
 }
@@ -179,9 +195,9 @@ func (r BigRat) toType(op string, conf *config.Config, which valueType) Value {
 		f := new(big.Float).SetPrec(conf.FloatPrec()).SetRat(r.Rat)
 		return BigFloat{f}
 	case complexType:
-		return newComplex(r, Int(0))
+		return NewComplex(r, zero)
 	case vectorType:
-		return NewVector([]Value{r})
+		return oneElemVector(r)
 	case matrixType:
 		return NewMatrix([]int{1, 1}, []Value{r})
 	}
