@@ -182,7 +182,7 @@ func toBool(t Value) bool {
 func andBool(t Value) bool {
 	if v, ok := t.(Vector); ok {
 		for _, x := range v {
-			if x == Int(0) {
+			if x == zero {
 				return false
 			}
 		}
@@ -201,16 +201,16 @@ func init() {
 			whichType:   binaryArithType,
 			fn: [numType]binaryFn{
 				intType: func(c Context, u, v Value) Value {
-					return newComplex(u, v)
+					return NewComplex(u, v)
 				},
 				bigIntType: func(c Context, u, v Value) Value {
-					return newComplex(u, v)
+					return NewComplex(u, v)
 				},
 				bigRatType: func(c Context, u, v Value) Value {
-					return newComplex(u, v)
+					return NewComplex(u, v)
 				},
 				bigFloatType: func(c Context, u, v Value) Value {
-					return newComplex(u, v)
+					return NewComplex(u, v)
 				},
 			},
 		},
@@ -355,6 +355,32 @@ func init() {
 			},
 		},
 
+		{ // Matrix division and vector projection.
+			name:        "mdiv",
+			elementwise: false,
+			whichType:   binaryArithType,
+			fn: [numType]binaryFn{
+				vectorType: func(c Context, u, v Value) Value {
+					// Projection of u onto v
+					uu, vv := u.(Vector), v.(Vector)
+					if len(uu) != len(vv) {
+						Errorf("mismatched lengths %d, %d in vector mdiv", len(uu), len(vv))
+					}
+					for i := range uu {
+						if whichType(uu[i]) >= complexType || whichType(vv[i]) >= complexType {
+							Errorf("non-real element in vector mdiv")
+						}
+					}
+					num := innerProduct(c, uu, "+", "*", vv)
+					den := innerProduct(c, vv, "+", "*", vv)
+					return c.EvalBinary(num, "/", den)
+				},
+				matrixType: func(c Context, u, v Value) Value {
+					return innerProduct(c, v.(*Matrix).inverse(c), "+", "*", u)
+				},
+			},
+		},
+
 		{ // Euclidean integer division.
 			name:        "div",
 			elementwise: true,
@@ -372,20 +398,15 @@ func init() {
 			},
 		},
 
-		{ // Euclidean integer modulus.
+		{ // Euclidean integer modulus, generalized.
 			name:        "mod",
 			elementwise: true,
-			whichType:   divType, // Use BigInts to avoid the analysis here.
+			whichType:   binaryArithType,
 			fn: [numType]binaryFn{
-				bigIntType: func(c Context, u, v Value) Value {
-					if v.(BigInt).Sign() == 0 {
-						Errorf("modulo by zero")
-					}
-					return binaryBigIntOp(u, (*big.Int).Mod, v) // Euclidan modulo.
-				},
-				bigRatType:   nil, // Not defined for rationals. Use mod.
-				bigFloatType: nil,
-				complexType:  nil,
+				intType:      mod,
+				bigIntType:   mod,
+				bigRatType:   mod,
+				bigFloatType: mod,
 			},
 		},
 
@@ -585,8 +606,8 @@ func init() {
 				},
 				complexType: func(c Context, u, v Value) Value {
 					i, j := u.(Complex), v.(Complex)
-					if c.EvalBinary(i.real, "==", j.real) == Int(0) {
-						return Int(0)
+					if c.EvalBinary(i.real, "==", j.real) == zero {
+						return zero
 					}
 					return c.EvalBinary(i.imag, "==", j.imag)
 				},
@@ -618,8 +639,8 @@ func init() {
 				},
 				complexType: func(c Context, u, v Value) Value {
 					i, j := u.(Complex), v.(Complex)
-					if c.EvalBinary(i.real, "!=", j.real) == Int(1) {
-						return Int(1)
+					if c.EvalBinary(i.real, "!=", j.real) == one {
+						return one
 					}
 					return c.EvalBinary(i.imag, "!=", j.imag)
 				},
@@ -894,11 +915,15 @@ func init() {
 					// If A is a vector, the elements of A align with B.
 					A, B := u.(Vector), v.(Vector)
 					if len(A) == 0 || len(B) == 0 {
-						return Int(0)
+						return zero
+					}
+					if allChars(A) {
+						// Special case for times.
+						return decodeTime(c, A, B)
 					}
 					if len(A) == 1 || len(B) == 1 || len(A) == len(B) {
-						result := Value(Int(0))
-						prod := Value(Int(1))
+						result := Value(zero)
+						prod := Value(one)
 						get := func(v Vector, i int) Value {
 							if len(v) == 1 {
 								return v[0]
@@ -923,7 +948,7 @@ func init() {
 				matrixType: func(c Context, u, v Value) Value {
 					A, B := u.(Vector), v.(*Matrix)
 					if len(A) != 1 && B.shape[0] != 1 && len(A) != B.shape[0] {
-						Errorf("decode of length %d and shape %s", len(A), NewIntVector(B.shape))
+						Errorf("decode of length %d and shape %s", len(A), NewIntVector(B.shape...))
 					}
 					shape := B.shape[1:]
 					elems := make([]Value, len(B.data)/B.shape[0])
@@ -939,8 +964,8 @@ func init() {
 					}
 					pfor(true, n, len(elems), func(lo, hi int) {
 						for j := lo; j < hi; j++ {
-							result := Value(Int(0))
-							prod := Value(Int(1))
+							result := Value(zero)
+							prod := Value(one)
 							for i := n - 1; i >= 0; i-- {
 								Bslice := B.data
 								if B.shape[0] > 1 {
@@ -975,9 +1000,14 @@ func init() {
 					// of how modulo arithmetic works.
 					const op = "encode"
 					A, B := u.(Vector), v.(Vector)
+					if allChars(A) {
+						// Special case for times.
+						return encodeTime(c, A, B)
+					}
 					// Scalar.
 					if len(A) == 1 && len(B) == 1 {
-						return emod(op, c, B[0], A[0])
+						_, rem := QuoRem(op, c, B[0], A[0])
+						return rem
 					}
 					// Vector.
 					if len(B) == 1 {
@@ -985,9 +1015,9 @@ func init() {
 						elems := make([]Value, len(A))
 						b := B[0]
 						for i := len(A) - 1; i >= 0; i-- {
-							a := A[i]
-							elems[i] = emod(op, c, b, a)
-							b = ediv(op, c, b, a)
+							quo, rem := QuoRem(op, c, b, A[i])
+							elems[i] = rem
+							b = quo
 						}
 						return NewVector(elems)
 					}
@@ -996,7 +1026,8 @@ func init() {
 						elems := make([]Value, len(B))
 						a := A[0]
 						for i := range B {
-							elems[i] = emod(op, c, B[i], a)
+							_, rem := QuoRem(op, c, B[i], a)
+							elems[i] = rem
 						}
 						return NewVector(elems)
 					}
@@ -1010,9 +1041,9 @@ func init() {
 						for j := lo; j < hi; j++ {
 							b := B[j]
 							for i := len(A) - 1; i >= 0; i-- {
-								a := A[i]
-								elems[j+i*len(B)] = emod(op, c, b, a)
-								b = ediv(op, c, b, a)
+								quo, rem := QuoRem(op, c, b, A[i])
+								elems[j+i*len(B)] = rem
+								b = quo
 							}
 						}
 					})
@@ -1027,9 +1058,9 @@ func init() {
 						for j := lo; j < hi; j++ {
 							b := B.data[j]
 							for i := len(A) - 1; i >= 0; i-- {
-								a := A[i]
-								elems[j+i*len(B.data)] = emod(op, c, b, a)
-								b = ediv(op, c, b, a)
+								quo, rem := QuoRem(op, c, b, A[i])
+								elems[j+i*len(B.data)] = rem
+								b = quo
 							}
 						}
 					})
@@ -1074,7 +1105,7 @@ func init() {
 						sortedA[i] = indexed{a, i + origin}
 					}
 					sort.SliceStable(sortedA, func(i, j int) bool {
-						return c.EvalBinary(sortedA[i].v, "<", sortedA[j].v) == Int(1)
+						return c.EvalBinary(sortedA[i].v, "<", sortedA[j].v) == one
 					})
 					indices := make([]Value, len(B))
 					work := 2 * (1 + int(math.Log2(float64(len(A)))))
@@ -1083,9 +1114,9 @@ func init() {
 							b := B[i]
 							indices[i] = Int(origin - 1)
 							pos := sort.Search(len(sortedA), func(j int) bool {
-								return c.EvalBinary(sortedA[j].v, ">=", b) == Int(1)
+								return c.EvalBinary(sortedA[j].v, ">=", b) == one
 							})
-							if pos < len(sortedA) && c.EvalBinary(sortedA[pos].v, "==", b) == Int(1) {
+							if pos < len(sortedA) && c.EvalBinary(sortedA[pos].v, "==", b) == one {
 								indices[i] = Int(sortedA[pos].index)
 							}
 						}
@@ -1096,7 +1127,7 @@ func init() {
 					A, B := u.(*Matrix), v.(*Matrix)
 					origin := c.Config().Origin()
 					if A.Rank()-1 > B.Rank() || !sameShape(A.shape[1:], B.shape[B.Rank()-(A.Rank()-1):]) {
-						Errorf("iota: mismatched shapes %s and %s", NewIntVector(A.shape), NewIntVector(B.shape))
+						Errorf("iota: mismatched shapes %s and %s", NewIntVector(A.shape...), NewIntVector(B.shape...))
 					}
 					// TODO: This is n^2. Use an algorithm similar to the Vector case, or perhaps use hashing.
 					// However, one of the n's is the dimension of a matrix, so it is likely to be small.
@@ -1240,37 +1271,63 @@ func init() {
 			},
 		},
 
+		{
+			name:      ",%",
+			whichType: atLeastVectorType,
+			fn: [numType]binaryFn{
+				vectorType: func(c Context, u, v Value) Value {
+					uu := u.(Vector)
+					uu = uu[:len(uu):len(uu)]
+					return append(uu, v.(Vector)...)
+				},
+				matrixType: func(c Context, u, v Value) Value {
+					return u.(*Matrix).catenateFirst(v.(*Matrix))
+				},
+			},
+		},
+
 		{
 			name:      "take",
 			whichType: vectorAndAtLeastVectorType,
 			fn: [numType]binaryFn{
 				vectorType: func(c Context, u, v Value) Value {
-					const bad = Error("bad count for take")
-					i := v.(Vector)
-					nv, ok := u.(Vector)
-					if !ok || len(nv) != 1 {
-						panic(bad)
+					uu := u.(Vector)
+					vv := v.(Vector)
+					if len(uu) != 1 {
+						// Need to expand to a matrix.
+						return NewMatrix([]int{len(vv)}, vv).take(c, uu)
 					}
-					n, ok := nv[0].(Int)
+					n, ok := uu[0].(Int) // Number of elements in result.
 					if !ok {
-						panic(bad)
+						Errorf("bad count %s in take", uu[0])
 					}
-					len := Int(len(i))
+					len := Int(len(vv)) // Length of rhs vector.
+					nElems := n
+					if n < 0 {
+						nElems = -nElems
+					}
+					elems := make([]Value, nElems)
+					fill := vv.fillValue()
 					switch {
 					case n < 0:
-						if -n > len {
-							panic(bad)
+						if nElems > len {
+							for i := 0; i < int(nElems-len); i++ {
+								elems[i] = fill
+							}
+							copy(elems[nElems-len:], vv)
+						} else {
+							copy(elems, vv[len-nElems:])
 						}
-						i = i[len+n : len : len]
 					case n == 0:
-						return NewVector(nil)
 					case n > 0:
-						if n > len {
-							panic(bad)
+						if nElems > len {
+							for i := len; i < nElems; i++ {
+								elems[i] = fill
+							}
 						}
-						i = i[0:n:n]
+						copy(elems, vv)
 					}
-					return i
+					return NewVector(elems)
 				},
 				matrixType: func(c Context, u, v Value) Value {
 					return v.(*Matrix).take(c, u.(Vector))
@@ -1283,31 +1340,30 @@ func init() {
 			whichType: vectorAndAtLeastVectorType,
 			fn: [numType]binaryFn{
 				vectorType: func(c Context, u, v Value) Value {
-					const bad = Error("bad count for drop")
-					i := v.(Vector)
-					nv, ok := u.(Vector)
-					if !ok || len(nv) != 1 {
-						panic(bad)
+					vv := v.(Vector)
+					uu, ok := u.(Vector)
+					if !ok || len(uu) != 1 {
+						Errorf("bad count %s in drop", u)
 					}
-					n, ok := nv[0].(Int)
+					n, ok := uu[0].(Int) // Number of elements in result.
 					if !ok {
-						panic(bad)
+						Errorf("bad count %s in drop", uu[0])
 					}
-					len := Int(len(i))
+					len := Int(len(vv)) // Length of rhs vector.
 					switch {
 					case n < 0:
 						if -n > len {
-							panic(bad)
+							return empty
 						}
-						i = i[0 : len+n]
+						vv = vv[0 : len+n]
 					case n == 0:
 					case n > 0:
 						if n > len {
-							panic(bad)
+							return empty
 						}
-						i = i[n:]
+						vv = vv[n:]
 					}
-					return i
+					return vv.Copy()
 				},
 				matrixType: func(c Context, u, v Value) Value {
 					return v.(*Matrix).drop(c, u.(Vector))
@@ -1378,7 +1434,7 @@ func init() {
 					i := u.(Vector)
 					j := v.(Vector)
 					if len(i) == 0 {
-						return NewVector(nil)
+						return empty
 					}
 					// All lhs values must be small integers.
 					var count int64
@@ -1406,20 +1462,20 @@ func init() {
 					}
 					result := make([]Value, 0, count)
 					jx := 0
-					var zero Value
+					var zeroVal Value
 					if j.AllChars() {
-						zero = Char(' ')
+						zeroVal = Char(' ')
 					} else {
-						zero = Int(0)
+						zeroVal = zero
 					}
 					for _, x := range i {
 						y := x.(Int)
 						switch {
 						case y == 0:
-							result = append(result, zero)
+							result = append(result, zeroVal)
 						case y < 0:
 							for y = -y; y > 0; y-- {
-								result = append(result, zero)
+								result = append(result, zeroVal)
 							}
 						default:
 							for ; y > 0; y-- {
@@ -1441,7 +1497,7 @@ func init() {
 					i := u.(Vector)
 					j := v.(Vector)
 					if len(i) == 0 {
-						return NewVector(nil)
+						return empty
 					}
 					// All lhs values must be small integers.
 					var count int64
@@ -1464,7 +1520,7 @@ func init() {
 						hm := int(howMany.(Int))
 						if hm < 0 {
 							hm = -hm
-							what = Int(0)
+							what = zero
 						}
 						for ; hm > 0; hm-- {
 							result = append(result, what)
@@ -1504,13 +1560,47 @@ func init() {
 			},
 		},
 
+		// Special cases that mix types, so don't promote them.
 		{
-			// Special case, handled in EvalBinary: don't modify types.
-			name:        "text",
-			elementwise: true,
-			whichType:   nil,
+			name:      "intersect",
+			whichType: noPromoteType,
 			fn: [numType]binaryFn{
-				0: fmtText,
+				intType:      intersect,
+				charType:     intersect,
+				bigIntType:   intersect,
+				bigRatType:   intersect,
+				bigFloatType: intersect,
+				complexType:  intersect,
+				vectorType:   intersect,
+			},
+		},
+
+		{
+			name:      "union",
+			whichType: noPromoteType,
+			fn: [numType]binaryFn{
+				intType:      union,
+				charType:     union,
+				bigIntType:   union,
+				bigRatType:   union,
+				bigFloatType: union,
+				complexType:  union,
+				vectorType:   union,
+			},
+		},
+
+		{
+			name:      "text",
+			whichType: noPromoteType,
+			fn: [numType]binaryFn{
+				intType:      fmtText,
+				charType:     fmtText,
+				bigIntType:   fmtText,
+				bigRatType:   fmtText,
+				bigFloatType: fmtText,
+				complexType:  fmtText,
+				vectorType:   fmtText,
+				matrixType:   fmtText,
 			},
 		},
 	}
