@@ -40,6 +40,10 @@ func (c Char) Eval(Context) Value {
 	return c
 }
 
+func (c Char) Copy() Value {
+	return c
+}
+
 func (c Char) Inner() Value {
 	return c
 }
@@ -49,7 +53,7 @@ func (c Char) toType(op string, conf *config.Config, which valueType) Value {
 	case charType:
 		return c
 	case vectorType:
-		return NewVector([]Value{c})
+		return oneElemVector(c)
 	case matrixType:
 		return NewMatrix([]int{1}, []Value{c})
 	}
@@ -79,7 +83,7 @@ func ParseString(s string) string {
 }
 
 // unquote is a simplified strconv.Unquote that treats ' and " equally.
-// Raw quotes are Go-like and bounded by ``.
+// Raw quotes are Go-like and bounded by “.
 // The return value is the string and a boolean rather than error, which
 // was almost always the same anyway.
 func unquote(s string) (t string, ok bool) {
