@@ -88,14 +88,6 @@ func whichType(v Value) valueType {
 }
 
 func (op *binaryOp) EvalBinary(c Context, u, v Value) Value {
-	if op.whichType == nil {
-		// At the moment, "text" is the only operator that leaves
-		// both arg types alone. Perhaps more will arrive.
-		if op.name != "text" {
-			Errorf("internal error: nil whichType")
-		}
-		return op.fn[0](c, u, v)
-	}
 	whichU, whichV := op.whichType(whichType(u), whichType(v))
 	conf := c.Config()
 	u = u.toType(op.name, conf, whichU)
@@ -115,6 +107,25 @@ func (op *binaryOp) EvalBinary(c Context, u, v Value) Value {
 	return fn(c, u, v)
 }
 
+// EvalCharEqual handles == and != in a special case:
+// If comparing a scalar against a Char, avoid the conversion.
+// The logic of type promotion in EvalBinary otherwise interferes with comparison
+// because it tries to force scalar types to be the same, and char doesn't convert to
+// any other type.
+func EvalCharEqual(u Value, isEqualOp bool, v Value) (Value, bool) {
+	uType, vType := whichType(u), whichType(v)
+	if uType != vType && uType < vectorType && vType < vectorType {
+		// Two different scalar types. If either is char, we know the answer now.
+		if uType == charType || vType == charType {
+			if isEqualOp {
+				return zero, true
+			}
+			return one, true
+		}
+	}
+	return nil, false
+}
+
 // Product computes a compound product, such as an inner product
 // "+.*" or outer product "o.*". The op is known to contain a
 // period. The operands are all at least vectors, and for inner product
@@ -228,7 +239,7 @@ func innerProduct(c Context, u Value, left, right string, v Value) Value {
 		// The result is has shape (-1 drop rho u), (1 drop rho v)
 		v := v.(*Matrix)
 		if u.Rank() < 1 || v.Rank() < 1 || u.shape[len(u.shape)-1] != v.shape[0] {
-			Errorf("inner product: mismatched shapes %s and %s", NewIntVector(u.shape), NewIntVector(v.shape))
+			Errorf("inner product: mismatched shapes %s and %s", NewIntVector(u.shape...), NewIntVector(v.shape...))
 		}
 		n := v.shape[0]
 		vstride := len(v.data) / n
@@ -265,7 +276,7 @@ func outerProduct(c Context, u Value, op string, v Value) Value {
 		v := v.(Vector)
 		m := Matrix{
 			shape: []int{len(u), len(v)},
-			data:  NewVector(make(Vector, len(u)*len(v))),
+			data:  make([]Value, len(u)*len(v)),
 		}
 		pfor(safeBinary(op), 1, len(m.data), func(lo, hi int) {
 			for x := lo; x < hi; x++ {
@@ -277,7 +288,7 @@ func outerProduct(c Context, u Value, op string, v Value) Value {
 		v := v.(*Matrix)
 		m := Matrix{
 			shape: append(u.Shape(), v.Shape()...),
-			data:  NewVector(make(Vector, len(u.Data())*len(v.Data()))),
+			data:  make([]Value, len(u.Data())*len(v.Data())),
 		}
 		vdata := v.Data()
 		udata := u.Data()
@@ -310,11 +321,11 @@ func Reduce(c Context, op string, v Value) Value {
 		return acc
 	case *Matrix:
 		if v.Rank() < 2 {
-			Errorf("shape for matrix is degenerate: %s", NewIntVector(v.shape))
+			Errorf("shape for matrix is degenerate: %s", NewIntVector(v.shape...))
 		}
 		stride := v.shape[v.Rank()-1]
 		if stride == 0 {
-			Errorf("shape for matrix is degenerate: %s", NewIntVector(v.shape))
+			Errorf("shape for matrix is degenerate: %s", NewIntVector(v.shape...))
 		}
 		shape := v.shape[:v.Rank()-1]
 		data := make(Vector, size(shape))
@@ -340,6 +351,41 @@ func Reduce(c Context, op string, v Value) Value {
 	panic("not reached")
 }
 
+// ReduceFirst computes a reduction such as +/% along
+// the first axis. The slash-percent has been removed.
+func ReduceFirst(c Context, op string, v Value) Value {
+	// We must be right associative; that is the grammar.
+	// -/1 2 3 == 1-2-3 is 1-(2-3) not (1-2)-3. Answer: 2.
+	m, ok := v.(*Matrix)
+	if !ok {
+		// Same as regular reduce.
+		return Reduce(c, op, v)
+	}
+	if v.Rank() < 2 {
+		Errorf("shape for matrix is degenerate: %s", NewIntVector(m.shape...))
+	}
+	stride := size(m.shape[1:m.Rank()])
+	if stride == 0 {
+		Errorf("shape for matrix is degenerate: %s", NewIntVector(m.shape...))
+	}
+	shape := m.shape[1:m.Rank()]
+	data := make(Vector, size(shape))
+	pfor(safeBinary(op), stride, len(data), func(lo, hi int) {
+		for i := lo; i < hi; i++ {
+			pos := i + len(m.data) - stride
+			acc := m.data[pos]
+			for j := pos - stride; j >= 0; j -= stride {
+				acc = c.EvalBinary(m.data[j], op, acc)
+			}
+			data[i] = acc
+		}
+	})
+	if len(shape) == 1 { // TODO: Matrix.shrink()?
+		return NewVector(data)
+	}
+	return NewMatrix(shape, data)
+}
+
 // Scan computes a scan of the op; the \ has been removed.
 // It gives the successive values of reducing op through v.
 // We must be right associative; that is the grammar.
@@ -367,18 +413,14 @@ func Scan(c Context, op string, v Value) Value {
 		return NewVector(values)
 	case *Matrix:
 		if v.Rank() < 2 {
-			Errorf("shape for matrix is degenerate: %s", NewIntVector(v.shape))
+			Errorf("shape for matrix is degenerate: %s", NewIntVector(v.shape...))
 		}
 		stride := v.shape[v.Rank()-1]
 		if stride == 0 {
-			Errorf("shape for matrix is degenerate: %s", NewIntVector(v.shape))
+			Errorf("shape for matrix is degenerate: %s", NewIntVector(v.shape...))
 		}
 		data := make(Vector, len(v.data))
-		nrows := 1
-		for i := 0; i < v.Rank()-1; i++ {
-			// Guaranteed by NewMatrix not to overflow.
-			nrows *= v.shape[i]
-		}
+		nrows := size(v.shape[:len(v.shape)-1])
 		pfor(safeBinary(op), stride, nrows, func(lo, hi int) {
 			for i := lo; i < hi; i++ {
 				index := i * stride
@@ -402,6 +444,32 @@ func Scan(c Context, op string, v Value) Value {
 	panic("not reached")
 }
 
+// ScanFirst computes a scan of the op along the first axis.
+// The backslash-percent has been removed.
+// It gives the successive values of reducing op through v.
+// We must be right associative; that is the grammar.
+func ScanFirst(c Context, op string, v Value) Value {
+	m, ok := v.(*Matrix)
+	if !ok {
+		// Same as regular reduce.
+		return Scan(c, op, v)
+	}
+	if m.Rank() < 2 {
+		Errorf("shape for matrix is degenerate: %s", NewIntVector(m.shape...))
+	}
+	stride := m.shape[len(m.shape)-1]
+	if stride == 0 {
+		Errorf("shape for matrix is degenerate: %s", NewIntVector(m.shape...))
+	}
+	// Simple but effective algorithm: Transpose twice. Better than one might
+	// think because transposition is O(size of matrix) and it also lines up
+	// the scan in memory order.
+	// TODO: Is it worth doing the ugly non-transpose bookkeeping?
+	m = Scan(c, op, m.transpose(c)).(*Matrix)
+	return m.transpose(c)
+
+}
+
 // unaryVectorOp applies op elementwise to i.
 func unaryVectorOp(c Context, op string, i Value) Value {
 	u := i.(Vector)
@@ -514,6 +582,11 @@ func binaryMatrixOp(c Context, i Value, op string, j Value) Value {
 	return NewMatrix(shape, NewVector(n))
 }
 
+// IsScalarType reports whether u is an actual scalar, an int or float etc.
+func IsScalarType(v Value) bool {
+	return whichType(v) < vectorType
+}
+
 // isScalar reports whether u is a 1x1x1x... item, that is, a scalar promoted to matrix.
 func isScalar(u *Matrix) bool {
 	for _, dim := range u.shape {
@@ -622,26 +695,137 @@ func isTrue(fnName string, v Value) bool {
 	}
 }
 
-// emod is a restricted form of Euclidean integer modulus.
-// Used by encode, and only works for integers.
-func emod(op string, c Context, a, b Value) Value {
-	if z, ok := b.(Int); ok && z == 0 {
-		return a
-	}
-	aa := a.toType(op, c.Config(), bigIntType)
-	bb := b.toType(op, c.Config(), bigIntType)
-	return binaryBigIntOp(aa, (*big.Int).Mod, bb)
+// sgn is a wrapper for calling "sgn v".
+func sgn(c Context, v Value) int {
+	return int(c.EvalUnary("sgn", v).(Int))
 }
 
-// ediv is a restricted form of Euclidean integer division.
-// Used by encode, and only works for integers.
-func ediv(op string, c Context, a, b Value) Value {
-	if z, ok := b.(Int); ok && z == 0 {
-		return a
+// inverse returns 1/v for any scalar value, errors otherwise.
+func inverse(c Context, v Value) Value {
+	switch v := v.(type) {
+	case Int:
+		return v.inverse()
+	case BigInt:
+		return v.inverse()
+	case BigRat:
+		return v.inverse()
+	case BigFloat:
+		return v.inverse()
+	case Complex:
+		return v.inverse(c)
 	}
-	aa := a.toType(op, c.Config(), bigIntType)
-	bb := b.toType(op, c.Config(), bigIntType)
-	return binaryBigIntOp(aa, (*big.Int).Div, bb)
+	Errorf("inverse of non-scalar %s", v)
+	return zero
+}
+
+func mod(c Context, a, b Value) Value {
+	_, rem := QuoRem("mod", c, a, b)
+	return rem
+}
+
+// QuoRem uses Euclidean division to return the quotient and remainder for a/b.
+// The quotient will be an integer, possibly negative; the remainder is always positive
+// and may be fractional. Returned values satisfy the identity that
+//	quo = a div b  such that
+//	rem = a - b*quo  with 0 <= rem < |y|
+// See comment for math/big.Int.DivMod for details.
+// Exported for testing.
+func QuoRem(op string, c Context, a, b Value) (div, rem Value) {
+	if z, ok := b.shrink().(Int); ok && z == 0 { // If zero, it must be shrinkable to Int.
+		return zero, a
+	}
+	aT := whichType(a)
+	bT := whichType(b)
+	negX, negY := false, false
+	// The calculations all do the division on the absolute values,
+	// then restore sign and adjust if necessary afterwards.
+	switch typ, _ := binaryArithType(aT, bT); typ {
+	case intType:
+		x := int(a.(Int))
+		y := int(b.(Int))
+		if x < 0 {
+			x = -x
+			negX = true
+		}
+		if y < 0 {
+			y = -y
+			negY = true
+		}
+		quo := x / y
+		rem := x % y
+		if negX && rem != 0 {
+			rem = y - rem
+			quo++
+		}
+		if negX != negY {
+			quo = -quo
+		}
+		return Int(quo), Int(rem)
+	case bigIntType:
+		x := a.toType(op, c.Config(), bigIntType).(BigInt)
+		y := b.toType(op, c.Config(), bigIntType).(BigInt)
+		rem := big.NewInt(0)
+		quo := big.NewInt(0)
+		// This is the one case we don't need to work hard.
+		quo.DivMod(x.Int, y.Int, rem)
+		return BigInt{quo}.shrink(), BigInt{rem}.shrink()
+	case bigRatType:
+		x := a.toType(op, c.Config(), bigRatType).(BigRat).Rat
+		y := b.toType(op, c.Config(), bigRatType).(BigRat).Rat
+		if x.Sign() < 0 {
+			x = x.Set(x) // Copy x.
+			x.Neg(x)
+			negX = true
+		}
+		if y.Sign() < 0 {
+			y = y.Set(y) // Copy y.
+			y.Neg(y)
+			negY = true
+		}
+		quo := big.NewRat(1, 1).Quo(x, y)
+		num := big.NewInt(0).Set(quo.Num())
+		iquo := num.Quo(num, quo.Denom()) // Truncation of quotient to an integer.
+		// quo is the division, iquo is its integer truncation. Remainder is (quo-iquo)*y.
+		rem := quo.Sub(quo, big.NewRat(1, 1).SetInt(iquo))
+		rem.Mul(rem, y)
+		if negX && rem.Sign() != 0 {
+			rem.Sub(y, rem)
+			iquo.Add(iquo, bigIntOne.Int)
+		}
+		if negX != negY {
+			iquo.Neg(iquo)
+		}
+		return BigInt{iquo}.shrink(), BigRat{rem}.shrink()
+	case bigFloatType:
+		x := a.toType(op, c.Config(), bigFloatType).(BigFloat).Float
+		y := b.toType(op, c.Config(), bigFloatType).(BigFloat).Float
+		if x.Sign() < 0 {
+			x = x.Copy(x)
+			x.Neg(x)
+			negX = true
+		}
+		if y.Sign() < 0 {
+			y = y.Copy(y)
+			y.Neg(y)
+			negY = true
+		}
+		quo := big.NewFloat(0).Quo(x, y)
+		iquo, _ := quo.Int(nil)
+		// quo is the division, iquo is its integer truncation. Remainder is (quo-iquo)*y.
+		rem := quo.Sub(quo, big.NewFloat(0).SetInt(iquo))
+		rem.Mul(rem, y)
+		if negX && rem.Sign() != 0 {
+			rem.Sub(y, rem)
+			iquo.Add(iquo, bigIntOne.Int)
+		}
+		if negX != negY {
+			iquo.Neg(iquo)
+		}
+		return BigInt{iquo}.shrink(), BigFloat{rem}.shrink()
+	default:
+		Errorf("%s undefined for type %s", op, typ)
+	}
+	return zero, a
 }
 
 // EvalFunctionBody evaluates the list of expressions inside a function,
@@ -660,3 +844,33 @@ func EvalFunctionBody(context Context, fnName string, body []Expr) Value {
 	}
 	return v
 }
+
+// flatten returns a simple vector containing the scalar elements of v.
+func flatten(v Value) []Value {
+	if IsScalarType(v) {
+		return []Value{v}
+	}
+	switch v := v.(type) {
+	case Vector:
+		if v.allScalars() {
+			return v.Copy().(Vector)
+		}
+		elems := make([]Value, 0, len(v))
+		for _, elem := range v {
+			if IsScalarType(elem) {
+				elems = append(elems, elem)
+			} else {
+				elems = append(elems, flatten(elem)...)
+			}
+		}
+		return elems
+	case *Matrix:
+		return flatten(v.data)
+	}
+	panic("flatten: can't happen")
+}
+
+// box returns its argument wrapped into a one-element vector.
+func box(c Context, v Value) Value {
+	return NewVector([]Value{v})
+}
