@@ -6,6 +6,7 @@ package value
 
 import (
 	"fmt"
+	"math/big"
 	"strconv"
 
 	"robpike.io/ivy/config"
@@ -105,6 +106,17 @@ func (i Int) eExponent() int {
 	return exp
 }
 
+// inverse returns 1/i
+func (i Int) inverse() Value {
+	v := int64(i)
+	if i == 0 {
+		Errorf("inverse of zero")
+	}
+	return BigRat{
+		Rat: big.NewRat(0, 1).SetFrac64(1, v),
+	}.shrink()
+}
+
 // eFormat returns the %e/%E form of the number represented by the
 // string str, which is a decimal integer, scaled by 10**exp.
 func eFormat(verb byte, prec int, sign, str string, exp int) string {
@@ -136,6 +148,10 @@ func (i Int) Eval(Context) Value {
 	return i
 }
 
+func (i Int) Copy() Value {
+	return i
+}
+
 func (i Int) Inner() Value {
 	return i
 }
@@ -151,9 +167,9 @@ func (i Int) toType(op string, conf *config.Config, which valueType) Value {
 	case bigFloatType:
 		return bigFloatInt64(conf, int64(i))
 	case complexType:
-		return newComplex(i, Int(0))
+		return NewComplex(i, zero)
 	case vectorType:
-		return NewVector([]Value{i})
+		return oneElemVector(i)
 	case matrixType:
 		return NewMatrix([]int{1}, []Value{i})
 	}
