@@ -61,14 +61,14 @@ func self(c Context, v Value) Value {
 }
 
 func returnZero(c Context, v Value) Value {
-	return Int(0)
+	return zero
 }
 
 func realPhase(c Context, v Value) Value {
 	if isNegative(v) {
 		return BigFloat{newFloat(c).Set(floatPi)}
 	}
-	return Int(0)
+	return zero
 }
 
 // vectorSelf promotes v to type Vector.
@@ -80,7 +80,7 @@ func vectorSelf(c Context, v Value) Value {
 	case *Matrix:
 		Errorf("internal error: vectorSelf of matrix")
 	}
-	return NewVector([]Value{v})
+	return oneElemVector(v)
 }
 
 // floatValueSelf promotes v to type BigFloat, and wraps it as a value.
@@ -121,18 +121,6 @@ var IvyEval func(context Context, s string) Value
 
 var UnaryOps = make(map[string]UnaryOp)
 
-func factorial(n int64) *big.Int {
-	if n < 0 {
-		Errorf("negative value %d for factorial", n)
-	}
-	if n == 0 {
-		return big.NewInt(1)
-	}
-	fac := new(big.Int)
-	fac.MulRange(1, n)
-	return fac
-}
-
 func init() {
 	ops := []*unaryOp{
 		{
@@ -160,21 +148,55 @@ func init() {
 			elementwise: true,
 			fn: [numType]unaryFn{
 				intType: func(c Context, v Value) Value {
-					return newComplex(Int(0), v)
+					return NewComplex(zero, v)
 				},
 				bigIntType: func(c Context, v Value) Value {
-					return newComplex(Int(0), v)
+					return NewComplex(zero, v)
 				},
 				bigRatType: func(c Context, v Value) Value {
-					return newComplex(Int(0), v)
+					return NewComplex(zero, v)
 				},
 				bigFloatType: func(c Context, v Value) Value {
-					return newComplex(Int(0), v)
+					return NewComplex(zero, v)
 				},
 				complexType: func(c Context, v Value) Value {
 					// Multiply by i.
 					u := v.(Complex)
-					return newComplex(c.EvalUnary("-", u.imag), u.real)
+					return NewComplex(c.EvalUnary("-", u.imag), u.real)
+				},
+			},
+		},
+
+		{
+			name: "split",
+			fn: [numType]unaryFn{
+				intType:      self,
+				bigIntType:   self,
+				bigRatType:   self,
+				bigFloatType: self,
+				complexType:  self,
+				// TODO: Vector?
+				matrixType: func(c Context, v Value) Value {
+					return v.(*Matrix).split()
+				},
+			},
+		},
+
+		{
+			name: "mix",
+			fn: [numType]unaryFn{
+				intType:      self,
+				bigIntType:   self,
+				bigRatType:   self,
+				bigFloatType: self,
+				complexType:  self,
+				vectorType: func(c Context, v Value) Value {
+					vv := v.(Vector)
+					return NewMatrix([]int{len(vv)}, vv).mix(c).shrink()
+
+				},
+				matrixType: func(c Context, v Value) Value {
+					return v.(*Matrix).mix(c)
 				},
 			},
 		},
@@ -219,37 +241,47 @@ func init() {
 			elementwise: true,
 			fn: [numType]unaryFn{
 				intType: func(c Context, v Value) Value {
-					i := int64(v.(Int))
-					if i == 0 {
-						Errorf("division by zero")
-					}
-					return BigRat{
-						Rat: big.NewRat(0, 1).SetFrac64(1, i),
-					}.shrink()
+					return v.(Int).inverse()
 				},
 				bigIntType: func(c Context, v Value) Value {
-					// Zero division cannot happen for unary.
-					return BigRat{
-						Rat: big.NewRat(0, 1).SetFrac(bigIntOne.Int, v.(BigInt).Int),
-					}.shrink()
+					return v.(BigInt).inverse()
 				},
 				bigRatType: func(c Context, v Value) Value {
-					// Zero division cannot happen for unary.
-					r := v.(BigRat)
-					return BigRat{
-						Rat: big.NewRat(0, 1).SetFrac(r.Denom(), r.Num()),
-					}.shrink()
+					return v.(BigRat).inverse()
 				},
 				bigFloatType: func(c Context, v Value) Value {
-					// Zero division cannot happen for unary.
-					f := v.(BigFloat)
-					one := new(big.Float).SetPrec(c.Config().FloatPrec()).SetInt64(1)
-					return BigFloat{
-						Float: one.Quo(one, f.Float),
-					}.shrink()
+					return v.(BigFloat).inverse()
 				},
 				complexType: func(c Context, v Value) Value {
-					return v.(Complex).recip(c)
+					return v.(Complex).inverse(c)
+				},
+			},
+		},
+
+		{
+			name:        "inv",
+			elementwise: false,
+			fn: [numType]unaryFn{
+				intType: func(c Context, v Value) Value {
+					return v.(Int).inverse()
+				},
+				bigIntType: func(c Context, v Value) Value {
+					return v.(BigInt).inverse()
+				},
+				bigRatType: func(c Context, v Value) Value {
+					return v.(BigRat).inverse()
+				},
+				bigFloatType: func(c Context, v Value) Value {
+					return v.(BigFloat).inverse()
+				},
+				complexType: func(c Context, v Value) Value {
+					return v.(Complex).inverse(c)
+				},
+				vectorType: func(c Context, v Value) Value {
+					return v.(Vector).inverse(c)
+				},
+				matrixType: func(c Context, v Value) Value {
+					return v.(*Matrix).inverse(c)
 				},
 			},
 		},
@@ -277,6 +309,23 @@ func init() {
 				bigFloatType: func(c Context, v Value) Value {
 					return Int(v.(BigFloat).Sign())
 				},
+				complexType: func(c Context, v Value) Value {
+					return v.(Complex).Signum(c).shrink()
+				},
+			},
+		},
+		{
+			name:        "conj",
+			elementwise: true,
+			fn: [numType]unaryFn{
+				intType:      self,
+				bigIntType:   self,
+				bigRatType:   self,
+				bigFloatType: self,
+				complexType: func(c Context, v Value) Value {
+					u := v.(Complex)
+					return NewComplex(u.real, c.EvalUnary("-", u.imag)).shrink()
+				},
 			},
 		},
 
@@ -449,6 +498,9 @@ func init() {
 					}
 					return BigInt{i}.shrink()
 				},
+				complexType: func(c Context, v Value) Value {
+					return v.(Complex).floor(c).shrink()
+				},
 			},
 		},
 
@@ -493,6 +545,9 @@ func init() {
 					}
 					return BigInt{i}.shrink()
 				},
+				complexType: func(c Context, v Value) Value {
+					return v.(Complex).ceil(c).shrink()
+				},
 			},
 		},
 
@@ -512,6 +567,43 @@ func init() {
 					copy(n, data)
 					return NewVector(n)
 				},
+				vectorType: func(c Context, v Value) Value {
+					// Produce a matrix of coordinates.
+					vv := v.(Vector)
+					if len(vv) == 0 {
+						return empty
+					}
+					nElems := 1
+					shape := make([]int, len(vv))
+					for i, coord := range vv {
+						c, ok := coord.(Int)
+						if !ok || c < 0 || maxInt < c {
+							Errorf("bad coordinate in iota %s", coord)
+						}
+						shape[i] = int(c)
+						nElems *= int(c)
+						if nElems > maxInt {
+							Errorf("shape too large in iota %s", vv)
+						}
+					}
+					origin := c.Config().Origin()
+					elems := make([]Value, nElems)
+					counter := make([]int, len(vv))
+					for i := range counter {
+						counter[i] = origin
+					}
+					for i := range elems {
+						elems[i] = NewIntVector(counter...)
+						for axis := len(counter) - 1; axis >= 0; axis-- {
+							counter[axis]++
+							if counter[axis]-origin < shape[axis] {
+								break
+							}
+							counter[axis] = origin
+						}
+					}
+					return NewMatrix(shape, elems)
+				},
 			},
 		},
 
@@ -519,28 +611,92 @@ func init() {
 			name: "rho",
 			fn: [numType]unaryFn{
 				intType: func(c Context, v Value) Value {
-					return Int(0)
+					return empty
 				},
 				charType: func(c Context, v Value) Value {
-					return Int(0)
+					return empty
 				},
 				bigIntType: func(c Context, v Value) Value {
-					return Int(0)
+					return empty
 				},
 				bigRatType: func(c Context, v Value) Value {
-					return Int(0)
+					return empty
 				},
 				bigFloatType: func(c Context, v Value) Value {
-					return Int(0)
+					return empty
 				},
 				complexType: func(c Context, v Value) Value {
-					return Int(0)
+					return empty
+				},
+				vectorType: func(c Context, v Value) Value {
+					return NewIntVector(len(v.(Vector)))
+				},
+				matrixType: func(c Context, v Value) Value {
+					return NewIntVector(v.(*Matrix).shape...)
+				},
+			},
+		},
+
+		{
+			name: "count",
+			fn: [numType]unaryFn{
+				intType: func(c Context, v Value) Value {
+					return one
+				},
+				charType: func(c Context, v Value) Value {
+					return one
+				},
+				bigIntType: func(c Context, v Value) Value {
+					return one
+				},
+				bigRatType: func(c Context, v Value) Value {
+					return one
+				},
+				bigFloatType: func(c Context, v Value) Value {
+					return one
+				},
+				complexType: func(c Context, v Value) Value {
+					return one
 				},
 				vectorType: func(c Context, v Value) Value {
 					return Int(len(v.(Vector)))
 				},
 				matrixType: func(c Context, v Value) Value {
-					return NewIntVector(v.(*Matrix).shape)
+					m := v.(*Matrix)
+					if len(m.shape) == 0 {
+						return zero
+					}
+					return Int(m.shape[0])
+				},
+			},
+		},
+
+		{
+			name: "flatten",
+			fn: [numType]unaryFn{
+				intType: func(c Context, v Value) Value {
+					return oneElemVector(v)
+				},
+				charType: func(c Context, v Value) Value {
+					return oneElemVector(v)
+				},
+				bigIntType: func(c Context, v Value) Value {
+					return oneElemVector(v)
+				},
+				bigRatType: func(c Context, v Value) Value {
+					return oneElemVector(v)
+				},
+				bigFloatType: func(c Context, v Value) Value {
+					return oneElemVector(v)
+				},
+				complexType: func(c Context, v Value) Value {
+					return oneElemVector(v)
+				},
+				vectorType: func(c Context, v Value) Value {
+					return NewVector(flatten(v.(Vector)))
+				},
+				matrixType: func(c Context, v Value) Value {
+					return c.EvalUnary("flatten", NewVector(v.(*Matrix).data))
 				},
 			},
 		},
@@ -608,7 +764,7 @@ func init() {
 					return v.(Vector).reverse()
 				},
 				matrixType: func(c Context, v Value) Value {
-					m := v.(*Matrix).Copy()
+					m := v.(*Matrix).Copy().(*Matrix)
 					if m.Rank() == 0 {
 						return m
 					}
@@ -641,7 +797,7 @@ func init() {
 					return v.(Vector).reverse()
 				},
 				matrixType: func(c Context, v Value) Value {
-					m := v.(*Matrix).Copy()
+					m := v.(*Matrix).Copy().(*Matrix)
 					if m.Rank() == 0 {
 						return m
 					}
@@ -908,7 +1064,8 @@ func init() {
 					if !text.AllChars() {
 						Errorf("ivy: value is not a vector of char")
 					}
-					return IvyEval(c, text.makeString(c.Config(), false))
+					str, _ := text.oneLineSprint(c.Config(), !withParens, !withSpaces)
+					return IvyEval(c, str)
 				},
 			},
 		},
@@ -923,10 +1080,74 @@ func init() {
 				bigFloatType: floatValueSelf,
 				complexType: func(c Context, v Value) Value {
 					u := v.(Complex)
-					return newComplex(floatValueSelf(c, u.real), floatValueSelf(c, u.imag))
+					return NewComplex(floatValueSelf(c, u.real), floatValueSelf(c, u.imag))
 				},
 			},
 		},
+
+		{
+			name:        "unique",
+			elementwise: false,
+			fn: [numType]unaryFn{
+				intType:      unique,
+				charType:     unique,
+				bigIntType:   unique,
+				bigRatType:   unique,
+				bigFloatType: unique,
+				complexType:  unique,
+				vectorType:   unique,
+			},
+		},
+
+		{
+			name:        "box",
+			elementwise: false,
+			fn: [numType]unaryFn{
+				intType:      box,
+				charType:     box,
+				bigIntType:   box,
+				bigRatType:   box,
+				bigFloatType: box,
+				complexType:  box,
+				vectorType:   box,
+				matrixType:   box,
+			},
+		},
+
+		{
+			name:        "first",
+			elementwise: false,
+			fn: [numType]unaryFn{
+				intType:      self,
+				charType:     self,
+				bigIntType:   self,
+				bigRatType:   self,
+				bigFloatType: self,
+				complexType:  self,
+				vectorType: func(c Context, v Value) Value {
+					u := v.(Vector)
+					if len(u) == 0 {
+						return zero // TODO: If we add prototypes, this is a place it matters.
+					}
+					return u[0]
+				},
+				matrixType: func(c Context, v Value) Value {
+					u := v.(*Matrix).data
+					if len(u) == 0 {
+						return zero // TODO: If we add prototypes, this is a place it matters.
+					}
+					return u[0]
+				},
+			},
+		},
+
+		{
+			name:        "sys",
+			elementwise: false,
+			fn: [numType]unaryFn{
+				vectorType: sys, // Expect a vector of chars.
+			},
+		},
 	}
 
 	for _, op := range ops {
