@@ -20,8 +20,13 @@ type Value interface {
 	// All user output should call Sprint instead.
 	String() string
 	Sprint(*config.Config) string
+
+	// Eval evaluates (simplifies) the Value.
 	Eval(Context) Value
 
+	// Copy returns a copy of the Value.
+	Copy() Value
+
 	// Inner retrieves the value, without evaluation. But for Assignments,
 	// it returns the right-hand side.
 	Inner() Value
@@ -65,7 +70,7 @@ func parseTwo(conf *config.Config, s string) (Value, Value, string, error) {
 		sep = "/"
 		typ = "rational"
 	} else {
-		return Int(0), Int(0), "", nil
+		return zero, zero, "", nil
 	}
 	elems = strings.Split(s, sep)
 	if len(elems) != 2 || elems[0] == "" || elems[1] == "" {
@@ -91,7 +96,7 @@ func Parse(conf *config.Config, s string) (Value, error) {
 	switch sep {
 	case "j":
 		// A complex.
-		return newComplex(v1, v2), nil
+		return NewComplex(v1, v2), nil
 	case "/":
 		// A rational. It's tricky.
 		// Common simple case.
